/* Mini Eric robot - using all available pins on Roboduino!
   - uses 2 modified servos for driving (960ms/360deg = 62.5 RPM)
   - uses 2 micro servos for a pan/tilt head
   - uses 2 micro servos for arms (120ms/60deg rotation)
   - uses 2 mini servos for shoulders (90ms/60deg rotation)
   - uses 1 standard servo for waist bend (160ms/60deg rotation)
   - uses one Ping))) ultrasonic sensor mounted on the head
   - will use AVRcam for blob detection, mounted on the head
   - uses one GP2D120 sensor mounted on a pan micro servo
   - uses 2 push buttons on arms
   - uses a battery monitor sensor
   - uses a piezo speaker to play melodies
   - uses a software serial port to talk to a serial LCD
*/

// Includes
#include <SoftwareSerial.h>
#include <ServoTimeTimer1.h>
#include <SoftwareServo.h>
#include <EEPROM.h>

// LCD pins
#define txPin 2         // digital pin 2
#define rxPin 13        // digital pin 13, not used for the LCD

// Servo pins
#define ScanPin 3       // digital pin 3, PWM pin
#define RarmPin 4       // digital pin 4
#define RshoulderPin 5  // digital pin 5, PWM pin
#define LshoulderPin 6  // digital pin 6, PWM pin
#define TiltPin 7       // digital pin 7
#define PanPin 8        // digital pin 8
#define RmotorPin 9     // digital pin  9, PWM pin
#define LmotorPin 10    // digital pin 10, PWM pin
#define WaistPin 11     // digital pin 11, PWM pin
#define LarmPin 12      // digital pin 12

// Media pins
#define ledPin 13       // digital pin 13                         
#define speakerOut 14   // digital pin 14 (analog pin 0)

// Sensor pins
#define PingPin 15      // digital pin 15 (analog pin 1)
#define IR_Pin  16 // digital pin 16 (analog pin 2)
#define SharpPin 3      // analog pin 3
#define TouchPin 4      // analog pin 4
#define BattSensor 5    // analog pin 5

// Remote control buttons
#define btnPower   149
#define btnMute    148
#define btnPrevCh  187
#define btnDown    145
#define btnUp      144
#define btnLeft    146
#define btnRight   147
#define btnPlay    1434
#define btnStop    1432
#define btnRecord  1437
#define btnPause   1433
#define btnRew     1435
#define btnFwd     1436
#define btnInfo    186
#define btnSleep   182
#define btnInput   165
#define btnEnter   139
#define btn0       137
#define btn1       128
#define btn2       129
#define btn3       130
#define btn4       131
#define btn5       132
#define btn6       133
#define btn7       134
#define btn8       135
#define btn9       136

// Servos
ServoTimeTimer1 Lmotor;
ServoTimeTimer1 Rmotor;
Servo Scan;
Servo Waist;
Servo Lshoulder;
Servo Rshoulder;
Servo Larm;
Servo Rarm;
Servo Pan;
Servo Tilt;

// Software serial
SoftwareSerial mySerial =  SoftwareSerial(rxPin, txPin);

//-------------------------------------------------------
// servo variables
#define center  80
byte Speed = 200;
byte TurnSpeed = 50;
byte Acc = 10;
byte direction = 1;

byte oldPos[8] = {0, 65, 180, 180, 80, 85, 80, 80};

// melody vaiables
byte names[] = {'c', 'd', 'e', 'f', 'g', 'a', 'b', 'C'}; 
int tones[] = {1915, 1700, 1519, 1432, 1275, 1136, 1014, 956};
byte melody[] = "2d2a1f2c2d2a2d2c2f2d2a2c2d2a1f2c2d2a2a2g2p8p8p8p";
// count length: 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
//                                10                  20                  30
int count = 0;
int count2 = 0;
int count3 = 0;
#define MAX_COUNT  24
boolean statePin = LOW;

// remote control variables
#define start_bit  2200  // Start bit threshold (Microseconds)
#define bin_1  1000         // Binary 1 threshold (Microseconds)
#define bin_0  400              // Binary 0 threshold (Microseconds)

byte sv = 1;
boolean teachMode = 0;
byte pos = 80;
byte index = 1;
byte moveNumber = 0;
byte pos_delay = 0;
boolean move = 0;
boolean stored = 0;
//----------------------------------------------------------------
void setup() {
  //hardware serial
  Serial.begin(9600);
 
  //software serial for LCD
  pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  mySerial.begin(9600);
  LCDposition(1, 1);
  mySerial.print("Mini Eric");
  LCDposition(2, 5);
  mySerial.print("robot");
  delay(100);    //wait a little
  LCDclear();

  //media
  pinMode(ledPin, OUTPUT);
  pinMode(speakerOut, OUTPUT); 

  pinMode(IR_Pin, INPUT); // uses an analog pin so it has to be declared as digital in
 
  //servos
  Lmotor.attach(LmotorPin);
  Rmotor.attach(RmotorPin);
  Scan.attach(ScanPin);
  Waist.attach(WaistPin);
  Lshoulder.attach(LshoulderPin);
  Rshoulder.attach(RshoulderPin);
  Larm.attach(LarmPin);
  Rarm.attach(RarmPin);
  Pan.attach(PanPin);
  Tilt.attach(TiltPin);

  moveNumber = 0;
  Play_Move();    // Stand_By

      LCDclear();
      LCDposition(1, 1);
      mySerial.print("Waiting for    ");
      LCDposition(2, 1);
      mySerial.print("remote input...");

}

//================================================================
// Main Loop
//================================================================
void loop(){
  Get_IR_Command();
  Servo::refresh();
} 
//================================================================
int Get_IR_Command() {
  int key = getIRKey();      //Fetch the key
  if (key != -1) {
      //LCDclear();
      //LCDposition(1, 1);
      //mySerial.print("Key: ");
      //mySerial.print(key, DEC);
  }
  // do something with the key...
  switch (key) {   
    case btnLeft:
      TurnLeft();
      break;
     
    case btnRight:   
      TurnRight();
      break;
     
    case btnUp:
      if (teachMode == 1){
          pos += 1;
          Servo_Position();
      }
      else {
          Forward();
      }
      break; 
     
    case btnDown:
      if (teachMode == 1){
          pos -= 1;
          Servo_Position();
      }
      else {
          Reverse();
      }
      break;

    case btnStop:
      Stop();
      break;

    case btnPause:
      moveNumber = 0;      //Stand_By();
      Play_Move();
      break;

    case btnPrevCh:
      moveNumber = 1;      //Rarm_Wave();
      Play_Move();
      break;

    case btnMute:
      Play_Music();
      break;

    case btnInfo:
      moveNumber = 2;      //Pick_Small_Object();
      Play_Move();
      break;

    case btnSleep:
      moveNumber = 3;      //Release();
      Play_Move();
      moveNumber = 0;      //Stand_By();
      Play_Move();
      break;

    case btnInput:
      LCDclear();
      if (teachMode != 1){
          LCDposition(1, 1);
          mySerial.print("Teach mode ON  ");
          teachMode = 1;
      }
      else {
          LCDposition(1, 1);
          mySerial.print("Teach mode OFF ");
          teachMode = 0;
      }
      break;

    case btnRecord:
      LCDclear();
      Store_Position();
      break;

    case btnPlay:
      LCDclear();
      Play_Move();
      break;

    case btnRew:
      LCDclear();
      Select_Servo();
      break;

    case btnFwd:
      LCDclear();
      Select_Move();
      break;

    case btnEnter:
      LCDclear();
      Record_Move();
      break;
  } 
  delay(20);
}

//--------------------------
int getIRKey() {
  int data[12];
  digitalWrite(ledPin, HIGH);    //Ok, i'm ready to recieve 
  while(pulseIn(IR_Pin, LOW) < start_bit) { //Wait for a start bit
  }
  for(int i=0;i<11;i++){
    data[i] = pulseIn(IR_Pin, LOW);   //Start measuring bits, we only want low pulses
  }
  digitalWrite(ledPin, LOW);          //Done receiving
 
  for(int i=0;i<11;i++) {       //Parse them
    if(data[i] > bin_1) {       //is it a 1?
 data[i] = 1;
    }  else {
 if(data[i] > bin_0) {       //is it a 0?
   data[i] = 0;
 } else {
  data[i] = 2;        //Flag the data as invalid; I don't know what it is!
 }
    }
  }

   for(int i=0;i<11;i++) {       //Pre-check data for errors
    if(data[i] > 1) {
 return -1;               //Return -1 on invalid data
    }
  }

  int result = 0;
  int seed = 1;
  for(int i=0;i<11;i++) {       //Convert bits to integer
    if(data[i] == 1) {
 result += seed;
    }
    seed = seed * 2;
  }
  return result;        //Return key number
}

//--------------------------------

void Record_Move() {
       // a move consists of: moveNumber, moveStart index, moveEnd index
       // moveStart is the index of the first servo that changes the position
       // moveEnd is the index of the last servo that changes the position
   if (move == 0) {
      moveNumber = EEPROM.read(0); //position 0 holds total number of recorded moves
      index = EEPROM.read(491+(2*moveNumber)); // last moveEnd index
      moveNumber += 1;  //add a new move
      index += 1;       //new move start index
      EEPROM.write((490+(2*moveNumber)), index); //write the moveStart index
      delay(5);
      move = 1; // set the flag
      LCDclear();
      LCDposition(1, 1);
      mySerial.print("Rec move ");
      mySerial.print(moveNumber, DEC);
      LCDposition(2, 1);
      mySerial.print("Start index ");
      mySerial.print(index, DEC);
      delay(50);
  }
   else {
     if (stored == 1) { //if we stored at least one servo position
       EEPROM.write(0, moveNumber); //write the new move to "0"
       delay(5);
       EEPROM.write(491+(2*moveNumber), index-1); // write the moveEnd index
       delay(5);
       move = 0;   //reset the flag
       stored = 0; //reset the flag
       LCDclear();
       LCDposition(1, 1);
       mySerial.print("Move recorded ");
       mySerial.print(moveNumber, DEC);
       LCDposition(2, 1);
       mySerial.print("End index ");
       mySerial.print(index-1, DEC);
       delay(50);
     }
   }
 }

void Select_Servo(){
  LCDposition(1, 1);
  mySerial.print("Servo: ");
  int sKey;
  do {
    sKey = getIRKey();      //Fetch the key
  } while (sKey == btnRew);
  switch (sKey) {
     case btn0:
       sv = 0;
       pos = 0;
       mySerial.print("Delay   ");
       break;
     case btn1:
       sv = 1;
       mySerial.print("Waist   ");
       break;
     case btn2:
       sv = 2;
       mySerial.print("Lshouldr");
       break;
     case btn3:
       sv = 3;
       mySerial.print("Rshouldr");
       break;
     case btn4:
       sv = 4;
       mySerial.print("Larm    ");
       break;
     case btn5:
       sv = 5;
       mySerial.print("Rarm    ");
       break;
     case btn6:
       sv = 6;
       mySerial.print("Tilt    ");
       break;
     case btn7:
       sv = 7;
       mySerial.print("Pan     ");
       break;
   }
}

void Servo_Position(){
   pos = constrain(pos, 1, 179);
   LCDposition(2, 1);
   mySerial.print("Position: ");
   mySerial.print(pos, DEC);
   mySerial.print("   ");
   switch (sv) {
     case 0:
       pos_delay = pos; //write number of step delays
       break;
     case 1:
       pos = constrain(pos, 66, 144);
       Waist.write(pos); //write the new position to the servo
       break;
     case 2:
       Lshoulder.write(pos); //write the new position to the servo
       break;
     case 3:
       Rshoulder.write(pos); //write the new position to the servo
       break;
     case 4:
       Larm.write(pos); //write the new position to the servo
       break;
     case 5:
       Rarm.write(pos); //write the new position to the servo
       break;
     case 6:
       Tilt.write(pos); //write the new position to the servo
       break;
     case 7:
       Pan.write(pos); //write the new position to the servo
       break;
   }
   Servo::refresh();
}

void Store_Position() {
   LCDposition(1, 1);
   mySerial.print("Storing         ");
   LCDposition(2, 1);
   mySerial.print("Positions       ");
  for (byte x=0; x<=7; x+=1){
    sv = x;
    switch (x) {
      case 0: 
        pos = pos_delay;
        EepromWrite();
        break;
      case 1: 
        pos = Waist.read();
        EepromWrite();
        break;
      case 2: 
        pos = Lshoulder.read();
        EepromWrite();
        break;
      case 3: 
        pos = Rshoulder.read();
        EepromWrite();
        break;
      case 4: 
        pos = Larm.read();
        EepromWrite();
        break;
      case 5: 
        pos = Rarm.read();
        EepromWrite();
        break;
      case 6: 
        pos = Tilt.read();
        EepromWrite();
        break;
      case 7: 
        pos = Pan.read();
        EepromWrite();
        break;
    }
  }
}

void EepromWrite() {
  if (pos != oldPos[sv]){         //if the servo did not move, don't save
      EEPROM.write(index, sv);    //write the servo number first
      delay(5);
      EEPROM.write(index+1, pos); //write the position
      delay(5);
      index += 2;
      if (sv > 0) oldPos[sv] = pos; //save as old position (if not a step delay)
      stored = 1;
   }
}

void Select_Move(){
  LCDclear();
  LCDposition(1, 1);
  mySerial.print("Select move: ");
  int newKey;
  do {
    newKey = getIRKey();      //Fetch the key
  } while (newKey == btnFwd);
  newKey = newKey - 127;
  if (newKey > 9) newKey = 0;
  mySerial.print(newKey, DEC);
  moveNumber = newKey;
}

void Play_Move() {
   LCDposition(1, 1);
   mySerial.print("Playing         ");
   LCDposition(2, 1);
   mySerial.print("Move ");
   mySerial.print(moveNumber, DEC);
   mySerial.print("          ");
  byte moveStart = EEPROM.read(490+(2*moveNumber));
  byte moveEnd = EEPROM.read(491+(2*moveNumber));
  for (byte i = moveStart; i <= moveEnd; i+=2){
    sv = EEPROM.read(i);
    pos = EEPROM.read(i+1);
    switch (sv) {
      case 0:
        for (int u=1; u<=pos; u++){
          StepDelay();
        }
        break;
      case 1:
        Waist.write(pos);
        break;
      case 2:
        Lshoulder.write(pos);
        break;
      case 3:
        Rshoulder.write(pos);
        break;
      case 4:
        Larm.write(pos);
        break;
      case 5:
        Rarm.write(pos);
        break;
      case 6:
        Tilt.write(pos);
        break;
      case 7:
        Pan.write(pos);
        break;
    }
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Stop(){
  switch (direction) {
    case 0:                           // direction = reverse
      for (int s=0; s <= Speed; s=s+Acc) {
        Lmotor.write(1315-Speed+s);   // rev 1015 - 1315 - 1615 fwd
        Rmotor.write(1330-Speed+s);   // rev 1030 - 1330 - 1630 fwd 
        delay(20);
      }
      break;
    case 1:                           // direction = forward
      for (int s=0; s<=Speed; s=s+Acc) {
        Lmotor.write(1315+Speed-s);   // rev 1015 - 1315 - 1615 fwd
        Rmotor.write(1330+Speed-s);   // rev 1030 - 1330 - 1630 fwd 
        delay(20);
      }
    break;
    case 2:                               // direction = left
      for (int s=0; s<=TurnSpeed; s=s+Acc) {
        Lmotor.write(1315+TurnSpeed-s);   // rev 1015 - 1315 - 1615 fwd
        Rmotor.write(1330-TurnSpeed+s);   // rev 1030 - 1330 - 1630 fwd 
        delay(20);
      }
      break;
    case 3:                               // direction = right
      for (int s=0; s<=TurnSpeed; s=s+Acc) {
        Lmotor.write(1315-TurnSpeed+s);   // rev 1015 - 1315 - 1615 fwd
        Rmotor.write(1330+TurnSpeed-s);   // rev 1030 - 1330 - 1630 fwd 
        delay(20);
      }
      break;
  }
}

void Forward(){
  direction = 1;
  for (int f=0; f<=Speed; f=f+Acc) {
    Lmotor.write(1315+f); // rev 1015 - 1315 - 1615 fwd
    Rmotor.write(1330+f); // rev 1030 - 1330 - 1630 fwd 
    delay(20);
  }
}

void Reverse(){
  direction = 0;
  for (int r=0; r<=Speed; r=r+Acc) {
    Lmotor.write(1315-r); // rev 1015 - 1315 - 1615 fwd
    Rmotor.write(1330-r); // rev 1030 - 1330 - 1630 fwd 
    delay(20);
  }
}

void TurnLeft(){
  direction = 2;
  for (int L=0; L<=Speed; L=L+Acc) {
    Lmotor.write(1315+L); // rev 1015 - 1315 - 1615 fwd
    Rmotor.write(1330-L); // rev 1030 - 1330 - 1630 fwd 
    delay(20);
  }
}

void TurnRight(){
  direction = 3;
  for (int R=0; R<=Speed; R=R+Acc) {
    Lmotor.write(1315-R); // rev 1015 - 1315 - 1615 fwd
    Rmotor.write(1330+R); // rev 1030 - 1330 - 1630 fwd 
    delay(20);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=

void StepDelay() {
    for (int t=0; t<=10; t++){
      Servo::refresh();
      delay(20);
    }
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void Play_Music() {
  digitalWrite(speakerOut, LOW);    
  for (count = 0; count < MAX_COUNT; count++) {
    statePin = !statePin;
    digitalWrite(ledPin, statePin);
    for (count3 = 0; count3 <= (melody[count*2] - 48) * 30; count3++) {
      for (count2=0;count2<8;count2++) {
        if (names[count2] == melody[count*2 + 1]) {      
          digitalWrite(speakerOut,  HIGH);
          delayMicroseconds(tones[count2]);
          digitalWrite(speakerOut, LOW);
          delayMicroseconds(tones[count2]);
        }
        if (melody[count*2 + 1] == 'p') {
          // make a pause of a certain size
          digitalWrite(speakerOut, 0);
          delayMicroseconds(500);
        }
      }
    }
  }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

 void LCDposition (byte line, byte column) {
  mySerial.print(line, BYTE);       // LCD line
  mySerial.print(column, BYTE);     // LCD column
}

void LCDclear () {
  mySerial.print(5, BYTE);          // select LCD command mode
  mySerial.print(1, BYTE);          // clear screen
}